#ifndef _CHRONOPTICS_TOF_CSF_WRITER_H_
#define _CHRONOPTICS_TOF_CSF_WRITER_H_

#include <chronoptics/tof/camera_config.h>
#include <chronoptics/tof/calibration.h>
#include <chronoptics/tof/data.h>
#include <chronoptics/tof/csf_reader.h>
#include <chronoptics/tof/camera.h>
#include <chronoptics/tof/gige_interface.h>
#include <chronoptics/tof/usb_interface.h>

#ifdef __cplusplus
extern "C" {
#endif

/** The csf writer class takes care of writing depth frames to disk.  The class
 * has to be created from either a Camera or csf reader, because it needs
 * information about the camera that was used to record the depth frames. 
 * Therefore you should only write depth frames coming from one camera into the
 * same writer

 */
typedef struct tof_csf_writer* tof_csf_writer_t;

/** Destruct tof_csf_writer */
TOF_EXPORT void tof_csf_writer_delete(tof_csf_writer_t ptr);

/** Construct csf writer from calibration and configuration
 * @param ptr Pointer to class
 * @param file_location Path to where the csf file should be created
 * @param camera_config Camera configuration
 * @param calibration Calibration
 * @param serial Serial number of camera
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_csf_writer_t tof_csf_writer_new(const char* file_location, const tof_camera_config_t camera_config, const tof_calibration_t calibration, const char* serial, tof_error_t *error);

/** Write frame to csf file
 * @param ptr Pointer to class
 * @param frame A data frame
 * @param error Pointer to error
 */
TOF_EXPORT void tof_csf_writer_write_frame(tof_csf_writer_t ptr, const tof_data_t frame, tof_error_t *error);

/** Create csf writer from csf reader
 * @param file_location Path to where the csf file should be created
 * @param reader A csf reader
 * @param error Pointer to error
 * @return csf writer
 */
TOF_EXPORT tof_csf_writer_t tof_create_csf_writer_reader(const char* file_location, const tof_csf_reader_t reader, tof_error_t *error);

/** Create csf writer from camera
 * @param file_location Path to where the csf file should be created
 * @param camera A camera
 * @param error Pointer to error
 * @return csf writer
 */
TOF_EXPORT tof_csf_writer_t tof_create_csf_writer_camera(const char* file_location, const tof_camera_t camera, tof_error_t *error);

/** Create csf writer from the GigE interface
 * @param file_location Path to where the csf file should be created
 * @param gige_interface A GigE interface
 * @param error Pointer to error
 * @return csf writer
 */
TOF_EXPORT tof_csf_writer_t tof_create_csf_writer_gige_interface(const char* file_location, const tof_gige_interface_t gige_interface, tof_error_t *error);

/** Create csf writer from the USB interface
 * @param file_location Path to where the csf file should be created
 * @param usb_interface A USB interface
 * @param error Pointer to error
 * @return csf writer
 */
TOF_EXPORT tof_csf_writer_t tof_create_csf_writer_usb_interface(const char* file_location, const tof_usb_interface_t usb_interface, tof_error_t *error);

#ifdef __cplusplus
}
#endif

#endif
