#ifndef _CHRONOPTICS_TOF_PROCESSING_CONFIG_H_
#define _CHRONOPTICS_TOF_PROCESSING_CONFIG_H_

#include <chronoptics/tof/error.h>

#ifdef __cplusplus
extern "C" {
#endif

/** The shape of which surrounding pixels are checked in the flying pixel
 * filter
 */
enum tof_flying_shape {
  TOF_FLYING_SHAPE_PLUS = 0, /** Four neighboring pixels in a plus shape */
  TOF_FLYING_SHAPE_BOX = 1, /** All eight neighboring pixels */
};

/** Mixed pixel sort mode. The mode you want to use depends on your
 * application.
 */
enum tof_mp_sort_mode {
  TOF_MP_SORT_MODE_DARKEST = 0, /** Choose darkest path */
  TOF_MP_SORT_MODE_SHORTEST = 1, /** Return the surface of a translucent object, resolve corners or checkerboard lens flare */
  TOF_MP_SORT_MODE_BRIGHTEST = 2, /** Mixed pixels between foreground and background, get the best SNR */
  TOF_MP_SORT_MODE_LONGEST = 3, /** Correct lens flare from close bright objects */
  TOF_MP_SORT_MODE_AGGRESSOR = 4, /** Correct lens flare */
  TOF_MP_SORT_MODE_ADAPTIVE_SHORT_V0 = 5, /** Adaptive shortest */
  TOF_MP_SORT_MODE_ADAPTIVE_SHORT_V1 = 6, /** Adaptive shortest version two */
  TOF_MP_SORT_MODE_AGGRESSOR_V2 = 7, /** Correct lens flare version two */
};

/** What kind of binning to do
 */
enum tof_binning_mode {
  TOF_BINNING_MODE_NORMAL = 0, /** Normal binning, a BxB grid averaged to a single pixel */
  TOF_BINNING_MODE_SMART = 1, /** Smart binning, take into account the standard deviation around a chosen pixel. Similar to local means but reduces the image size */
};

/** In what dimension the xyz is exported
 */
enum tof_xyz_dimension {
  TOF_XYZ_DIMENSION_MILLIMETER = 0, /** Export in millimeters */
  TOF_XYZ_DIMENSION_METER = 1, /** Export in meters */
};

/** Processing that can be done
 */
typedef struct tof_processing_config* tof_processing_config_t;

/** Destruct tof_processing_config */
TOF_EXPORT void tof_processing_config_delete(tof_processing_config_t ptr);

/** Create default processing config
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_processing_config_t tof_processing_config_new_default(tof_error_t *error);

/** Load processing config from disk
 * @param ptr Pointer to class
 * @param file_location Location of processing config
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_processing_config_t tof_processing_config_new_from_disk(const char* file_location, tof_error_t *error);

/** Write processing config to disk
 * @param ptr Pointer to class
 * @param file_location Location where to save the processing config
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_write(const tof_processing_config_t ptr, const char* file_location, tof_error_t *error);

/** Get whether processing on the gpu is enabled
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_gpu(const tof_processing_config_t ptr, tof_error_t *error);

/** Set whether processing on the gpu is enabled
 * @param ptr Pointer to class
 * @param gpu Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_gpu(tof_processing_config_t ptr, bool gpu, tof_error_t *error);

/** Get whether the calibration is applied
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_calibration_enabled(const tof_processing_config_t ptr, tof_error_t *error);

/** Set whether the calibration is applied
 * @param ptr Pointer to class
 * @param enabled Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_calibration_enabled(tof_processing_config_t ptr, bool enabled, tof_error_t *error);

/** Get whether phase unwrapping is enabled
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_phase_unwrapping_enabled(const tof_processing_config_t ptr, tof_error_t *error);

/** Set whether phase unwrapping is enabled
 * @param ptr Pointer to class
 * @param enabled Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_phase_unwrapping_enabled(tof_processing_config_t ptr, bool enabled, tof_error_t *error);

/** Get The maximum distance between ideal location and actual
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Max offset, range [0, 1]
 */
TOF_EXPORT double tof_processing_config_get_phase_unwrapping_max_offset(const tof_processing_config_t ptr, tof_error_t *error);

/** Set The maximum distance between ideal location and actual
 * @param ptr Pointer to class
 * @param max_offset Max offset, range [0, 1]
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_phase_unwrapping_max_offset(tof_processing_config_t ptr, double max_offset, tof_error_t *error);

/** Get wether mpi detect is enabled
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_mpi_detect_enabled(const tof_processing_config_t ptr, tof_error_t *error);

/** Set wether mpi detect is enabled
 * @param ptr Pointer to class
 * @param enabled Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_mpi_detect_enabled(tof_processing_config_t ptr, bool enabled, tof_error_t *error);

/** Get The amplitude ratio value
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The maximum ratio
 */
TOF_EXPORT float tof_processing_config_get_mpi_detect_aratio(const tof_processing_config_t ptr, tof_error_t *error);

/** Set The amplitude ratio value
 * @param ptr Pointer to class
 * @param aratio The maximum ratio
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_mpi_detect_aratio(tof_processing_config_t ptr, float aratio, tof_error_t *error);

/** Get the phase difference ratio
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The maximum phase difference
 */
TOF_EXPORT float tof_processing_config_get_mpi_detect_phi_diff(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the phase difference ratio
 * @param ptr Pointer to class
 * @param phi_diff The maximum phase difference
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_mpi_detect_phi_diff(tof_processing_config_t ptr, float phi_diff, tof_error_t *error);

/** Get wether mixed pixel is enabled
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_mixed_pixel_enabled(const tof_processing_config_t ptr, tof_error_t *error);

/** Set wether mixed pixel is enabled
 * @param ptr Pointer to class
 * @param enabled Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_mixed_pixel_enabled(tof_processing_config_t ptr, bool enabled, tof_error_t *error);

/** Get location of the mixed pixel look up table file
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return File location
 */
TOF_EXPORT const char* tof_processing_config_get_mixed_pixel_lut_file(const tof_processing_config_t ptr, tof_error_t *error);

/** Set location of the mixed pixel look up table file
 * @param ptr Pointer to class
 * @param file_location File location
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_mixed_pixel_lut_file(tof_processing_config_t ptr, const char* file_location, tof_error_t *error);

/** Get minimum amplitude that pixel needs to have to apply mixed pixel to it
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Minimum amplitude
 */
TOF_EXPORT double tof_processing_config_get_mixed_pixel_amp_min(const tof_processing_config_t ptr, tof_error_t *error);

/** Set minimum amplitude that pixel needs to have to apply mixed pixel to it
 * @param ptr Pointer to class
 * @param amp_min Minimum amplitude
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_mixed_pixel_amp_min(tof_processing_config_t ptr, double amp_min, tof_error_t *error);

/** Get what output is selected
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Sort mode
 */
TOF_EXPORT enum tof_mp_sort_mode tof_processing_config_get_mixed_pixel_sort_mode(const tof_processing_config_t ptr, tof_error_t *error);

/** Set what output is selected
 * @param ptr Pointer to class
 * @param sort_mode Sort mode
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_mixed_pixel_sort_mode(tof_processing_config_t ptr, enum tof_mp_sort_mode sort_mode, tof_error_t *error);

/** Get the amplitude ratio for detecting MPI
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Amplitude ratio
 */
TOF_EXPORT double tof_processing_config_get_mixed_pixel_aratio(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the amplitude ratio for detecting MPI
 * @param ptr Pointer to class
 * @param aratio Amplitude ratio
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_mixed_pixel_aratio(tof_processing_config_t ptr, double aratio, tof_error_t *error);

/** Get the phase difference for detecting MPI
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Phase difference
 */
TOF_EXPORT double tof_processing_config_get_mixed_pixel_phi_diff(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the phase difference for detecting MPI
 * @param ptr Pointer to class
 * @param phi_diff Phase difference
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_mixed_pixel_phi_diff(tof_processing_config_t ptr, double phi_diff, tof_error_t *error);

/** Get the threshold to classify a pixel as an aggressor
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Aggressor threshold
 */
TOF_EXPORT double tof_processing_config_get_mixed_pixel_aggressor_threshold(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the threshold to classify a pixel as an aggressor
 * @param ptr Pointer to class
 * @param threshold Aggressor threshold
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_mixed_pixel_aggressor_threshold(tof_processing_config_t ptr, double threshold, tof_error_t *error);

/** Get the minimum amount of bright pixels before a bright spot is classified
 * as an aggressor
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Minimum aggressor size
 */
TOF_EXPORT int32_t tof_processing_config_get_mixed_pixel_min_aggressor_size(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the minimum amount of bright pixels before a bright spot is classified
 * as an aggressor
 * @param ptr Pointer to class
 * @param size Minimum aggressor size
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_mixed_pixel_min_aggressor_size(tof_processing_config_t ptr, int32_t size, tof_error_t *error);

/** Get the maximum error when using three modulation frequencies
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The error
 */
TOF_EXPORT double tof_processing_config_get_mixed_pixel_max_error_3f(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the maximum error when using three modulation frequencies
 * @param ptr Pointer to class
 * @param max_error_3f The error
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_mixed_pixel_max_error_3f(tof_processing_config_t ptr, double max_error_3f, tof_error_t *error);

/** Get the maximum amplitude for adaptive sorting
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Maximum amplitude
 */
TOF_EXPORT double tof_processing_config_get_mixed_pixel_adaptive_amp_max(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the maximum amplitude for adaptive sorting
 * @param ptr Pointer to class
 * @param adaptive_amp_max Maximum amplitude
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_mixed_pixel_adaptive_amp_max(tof_processing_config_t ptr, double adaptive_amp_max, tof_error_t *error);

/** Get the minimum amplitude for adaptive sorting
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Minimum amplitude
 */
TOF_EXPORT double tof_processing_config_get_mixed_pixel_adaptive_amp_min(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the minimum amplitude for adaptive sorting
 * @param ptr Pointer to class
 * @param adaptive_amp_min Minimum amplitude
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_mixed_pixel_adaptive_amp_min(tof_processing_config_t ptr, double adaptive_amp_min, tof_error_t *error);

/** Get the minimum return amplitude ratio for adaptive sorting
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Minimum return ratio
 */
TOF_EXPORT double tof_processing_config_get_mixed_pixel_aratio_min(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the minimum return amplitude ratio for adaptive sorting
 * @param ptr Pointer to class
 * @param aratio_min Minimum return ratio
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_mixed_pixel_aratio_min(tof_processing_config_t ptr, double aratio_min, tof_error_t *error);

/** Get The maximum phase difference before a return is not classified as
 * aggressor
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Phase offset before not a aggressor return
 */
TOF_EXPORT double tof_processing_config_get_mixed_pixel_aggressor_phi_offset(const tof_processing_config_t ptr, tof_error_t *error);

/** Set The maximum phase difference before a return is not classified as
 * aggressor
 * @param ptr Pointer to class
 * @param aggressor_phi_offset Phase offset before not a aggressor return
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_mixed_pixel_aggressor_phi_offset(tof_processing_config_t ptr, double aggressor_phi_offset, tof_error_t *error);

/** Get whether hdr is enabled. High dynamic range allows you to separate depth
 * with high intensity difference. This is mostly useful when the sensor is
 * being saturated.
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_hdr_enabled(const tof_processing_config_t ptr, tof_error_t *error);

/** Set whether hdr is enabled. High dynamic range allows you to separate depth
 * with high intensity difference. This is mostly useful when the sensor is
 * being saturated.
 * @param ptr Pointer to class
 * @param enabled Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_hdr_enabled(tof_processing_config_t ptr, bool enabled, tof_error_t *error);

/** Get whether averaging is enabled. The average process allows you to average
 * multiple frames together at the cost of frame rate
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_average_enabled(const tof_processing_config_t ptr, tof_error_t *error);

/** Set whether averaging is enabled. The average process allows you to average
 * multiple frames together at the cost of frame rate
 * @param ptr Pointer to class
 * @param enabled Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_average_enabled(tof_processing_config_t ptr, bool enabled, tof_error_t *error);

/** Get the number of frames to average together
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Number of frames
 */
TOF_EXPORT size_t tof_processing_config_get_average_nframes(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the number of frames to average together
 * @param ptr Pointer to class
 * @param nframes Number of frames
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_average_nframes(tof_processing_config_t ptr, size_t nframes, tof_error_t *error);

/** Get whether the temporal filter is enabled. The temporal filter is a running
 * average as long as the values are within sigma standard deviations of each
 * other
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_temporal_enabled(const tof_processing_config_t ptr, tof_error_t *error);

/** Set whether the temporal filter is enabled. The temporal filter is a running
 * average as long as the values are within sigma standard deviations of each
 * other
 * @param ptr Pointer to class
 * @param enabled Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_temporal_enabled(tof_processing_config_t ptr, bool enabled, tof_error_t *error);

/** Get the number of standard deviations before resetting the running average
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The standard deviation
 */
TOF_EXPORT double tof_processing_config_get_temporal_sigma(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the number of standard deviations before resetting the running average
 * @param ptr Pointer to class
 * @param sigma The standard deviation
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_temporal_sigma(tof_processing_config_t ptr, double sigma, tof_error_t *error);

/** Get the maximum number of frames to average together
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Number of frames
 */
TOF_EXPORT size_t tof_processing_config_get_temporal_nframes(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the maximum number of frames to average together
 * @param ptr Pointer to class
 * @param nframes Number of frames
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_temporal_nframes(tof_processing_config_t ptr, size_t nframes, tof_error_t *error);

/** Get whether the gaussian blur is enabled. This is a standard gaussian blur
 * applied over the depth image
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_gaussian_enabled(const tof_processing_config_t ptr, tof_error_t *error);

/** Set whether the gaussian blur is enabled. This is a standard gaussian blur
 * applied over the depth image
 * @param ptr Pointer to class
 * @param enabled Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_gaussian_enabled(tof_processing_config_t ptr, bool enabled, tof_error_t *error);

/** Get the gaussian region size
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Region size, 3x3 and 5x5 are supported
 */
TOF_EXPORT size_t tof_processing_config_get_gaussian_size(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the gaussian region size
 * @param ptr Pointer to class
 * @param size Region size, 3x3 and 5x5 are supported
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_gaussian_size(tof_processing_config_t ptr, size_t size, tof_error_t *error);

/** Get the gaussian sigma
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The standard deviation
 */
TOF_EXPORT double tof_processing_config_get_gaussian_sigma(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the gaussian sigma
 * @param ptr Pointer to class
 * @param sigma The standard deviation
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_gaussian_sigma(tof_processing_config_t ptr, double sigma, tof_error_t *error);

/** Get whether the median filter is enabled. This is a standard median filter
 * applied over the depth image
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_median_enabled(const tof_processing_config_t ptr, tof_error_t *error);

/** Set whether the median filter is enabled. This is a standard median filter
 * applied over the depth image
 * @param ptr Pointer to class
 * @param enabled Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_median_enabled(tof_processing_config_t ptr, bool enabled, tof_error_t *error);

/** Get the median region size
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Region size, 3x3 and 5x5 are supported
 */
TOF_EXPORT size_t tof_processing_config_get_median_size(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the median region size
 * @param ptr Pointer to class
 * @param size Region size, 3x3 and 5x5 are supported
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_median_size(tof_processing_config_t ptr, size_t size, tof_error_t *error);

/** Get whether the bilateral filter is enabled. This filter averages the
 * current pixel with neighboring ones if they're within the specified standard
 * deviation
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_bilateral_enabled(const tof_processing_config_t ptr, tof_error_t *error);

/** Set whether the bilateral filter is enabled. This filter averages the
 * current pixel with neighboring ones if they're within the specified standard
 * deviation
 * @param ptr Pointer to class
 * @param enabled Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_bilateral_enabled(tof_processing_config_t ptr, bool enabled, tof_error_t *error);

/** Get the bilateral region size
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Region size, 3x3 and 5x5 are supported
 */
TOF_EXPORT size_t tof_processing_config_get_bilateral_size(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the bilateral region size
 * @param ptr Pointer to class
 * @param size Region size, 3x3 and 5x5 are supported
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_bilateral_size(tof_processing_config_t ptr, size_t size, tof_error_t *error);

/** Get the bilateral standard deviation
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The standard deviation
 */
TOF_EXPORT double tof_processing_config_get_bilateral_sigma(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the bilateral standard deviation
 * @param ptr Pointer to class
 * @param sigma The standard deviation
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_bilateral_sigma(tof_processing_config_t ptr, double sigma, tof_error_t *error);

/** Get Whether the local means filter is enabled.
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_local_means_enabled(const tof_processing_config_t ptr, tof_error_t *error);

/** Set Whether the local means filter is enabled.
 * @param ptr Pointer to class
 * @param enabled Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_local_means_enabled(tof_processing_config_t ptr, bool enabled, tof_error_t *error);

/** Get The local means region size
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Region size
 */
TOF_EXPORT size_t tof_processing_config_get_local_means_size(const tof_processing_config_t ptr, tof_error_t *error);

/** Set The local means region size
 * @param ptr Pointer to class
 * @param size Region size
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_local_means_size(tof_processing_config_t ptr, size_t size, tof_error_t *error);

/** Get The local means standard deviation for how close values are
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The stand deviation
 */
TOF_EXPORT double tof_processing_config_get_local_means_sigma(const tof_processing_config_t ptr, tof_error_t *error);

/** Set The local means standard deviation for how close values are
 * @param ptr Pointer to class
 * @param sigma The stand deviation
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_local_means_sigma(tof_processing_config_t ptr, double sigma, tof_error_t *error);

/** Get whether flying pixel filter is enabled. The flying pixel filters out
 * pixels when more than x edges are farther than x size away
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_flying_enabled(const tof_processing_config_t ptr, tof_error_t *error);

/** Set whether flying pixel filter is enabled. The flying pixel filters out
 * pixels when more than x edges are farther than x size away
 * @param ptr Pointer to class
 * @param enabled Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_flying_enabled(tof_processing_config_t ptr, bool enabled, tof_error_t *error);

/** Get the shape of which surrounding pixels are checked
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The shape
 */
TOF_EXPORT enum tof_flying_shape tof_processing_config_get_flying_shape(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the shape of which surrounding pixels are checked
 * @param ptr Pointer to class
 * @param shape The shape
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_flying_shape(tof_processing_config_t ptr, enum tof_flying_shape shape, tof_error_t *error);

/** Get the maximum distance allowed between surrounding pixels
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Maximum distance
 */
TOF_EXPORT size_t tof_processing_config_get_flying_distance(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the maximum distance allowed between surrounding pixels
 * @param ptr Pointer to class
 * @param distance Maximum distance
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_flying_distance(tof_processing_config_t ptr, size_t distance, tof_error_t *error);

/** Get the number of edges allowed to be larger than maximum distance
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Maximum edges
 */
TOF_EXPORT size_t tof_processing_config_get_flying_edges(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the number of edges allowed to be larger than maximum distance
 * @param ptr Pointer to class
 * @param edges Maximum edges
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_flying_edges(tof_processing_config_t ptr, size_t edges, tof_error_t *error);

/** Get whether the amplitude threshold filter is enabled. This filter filters
 * out pixels outside of the specified values
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_amp_threshold_enabled(const tof_processing_config_t ptr, tof_error_t *error);

/** Set whether the amplitude threshold filter is enabled. This filter filters
 * out pixels outside of the specified values
 * @param ptr Pointer to class
 * @param enabled Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_amp_threshold_enabled(tof_processing_config_t ptr, bool enabled, tof_error_t *error);

/** Get the minimum amplitude allowed
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Minimum value
 */
TOF_EXPORT double tof_processing_config_get_amp_threshold_min(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the minimum amplitude allowed
 * @param ptr Pointer to class
 * @param min Minimum value
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_amp_threshold_min(tof_processing_config_t ptr, double min, tof_error_t *error);

/** Get the maximum amplitude allowed
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Maximum value
 */
TOF_EXPORT double tof_processing_config_get_amp_threshold_max(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the maximum amplitude allowed
 * @param ptr Pointer to class
 * @param max Maximum value
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_amp_threshold_max(tof_processing_config_t ptr, double max, tof_error_t *error);

/** Get whether the distance threshold filter is enabled. This filter filters
 * out pixels outside of the specified values
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_dist_threshold_enabled(const tof_processing_config_t ptr, tof_error_t *error);

/** Set whether the distance threshold filter is enabled. This filter filters
 * out pixels outside of the specified values
 * @param ptr Pointer to class
 * @param enabled Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_dist_threshold_enabled(tof_processing_config_t ptr, bool enabled, tof_error_t *error);

/** Get the minimum distance allowed
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Minimum value
 */
TOF_EXPORT double tof_processing_config_get_dist_threshold_min(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the minimum distance allowed
 * @param ptr Pointer to class
 * @param min Minimum value
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_dist_threshold_min(tof_processing_config_t ptr, double min, tof_error_t *error);

/** Get the maximum distance allowed
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Maximum value
 */
TOF_EXPORT double tof_processing_config_get_dist_threshold_max(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the maximum distance allowed
 * @param ptr Pointer to class
 * @param max Maximum value
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_dist_threshold_max(tof_processing_config_t ptr, double max, tof_error_t *error);

/** Get how much to scale the radial value by
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Radial scale
 */
TOF_EXPORT float tof_processing_config_get_radial_scale(const tof_processing_config_t ptr, tof_error_t *error);

/** Set how much to scale the radial value by
 * @param ptr Pointer to class
 * @param scale Radial scale
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_radial_scale(tof_processing_config_t ptr, float scale, tof_error_t *error);

/** Get how much to add to the radial value
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Radial addition
 */
TOF_EXPORT float tof_processing_config_get_radial_add(const tof_processing_config_t ptr, tof_error_t *error);

/** Set how much to add to the radial value
 * @param ptr Pointer to class
 * @param addition Radial addition
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_radial_add(tof_processing_config_t ptr, float addition, tof_error_t *error);

/** Get how much to scale the intensity value by
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Intensity scale
 */
TOF_EXPORT float tof_processing_config_get_intensity_scale(const tof_processing_config_t ptr, tof_error_t *error);

/** Set how much to scale the intensity value by
 * @param ptr Pointer to class
 * @param scale Intensity scale
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_intensity_scale(tof_processing_config_t ptr, float scale, tof_error_t *error);

/** Get how much to add to the intensity value
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Intensity addition
 */
TOF_EXPORT float tof_processing_config_get_intensity_add(const tof_processing_config_t ptr, tof_error_t *error);

/** Set how much to add to the intensity value
 * @param ptr Pointer to class
 * @param addition Intensity addition
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_intensity_add(tof_processing_config_t ptr, float addition, tof_error_t *error);

/** Get the characterization addition component. The characterization function
 * computes the sigma for the temporal and bilateral filter
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return addition
 */
TOF_EXPORT double tof_processing_config_get_char_add(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the characterization addition component. The characterization function
 * computes the sigma for the temporal and bilateral filter
 * @param ptr Pointer to class
 * @param addition addition
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_char_add(tof_processing_config_t ptr, double addition, tof_error_t *error);

/** Get the characterization multiplication component. The characterization
 * function computes the sigma for the temporal and bilateral filter
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Multiplication
 */
TOF_EXPORT double tof_processing_config_get_char_mult(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the characterization multiplication component. The characterization
 * function computes the sigma for the temporal and bilateral filter
 * @param ptr Pointer to class
 * @param multiplication Multiplication
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_char_mult(tof_processing_config_t ptr, double multiplication, tof_error_t *error);

/** Get whether software binning is enabled
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_binning_enabled(const tof_processing_config_t ptr, tof_error_t *error);

/** Set whether software binning is enabled
 * @param ptr Pointer to class
 * @param enabled Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_binning_enabled(tof_processing_config_t ptr, bool enabled, tof_error_t *error);

/** Get the size of the area to bin, 2 would bin a 2x2 area
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The bin size
 */
TOF_EXPORT size_t tof_processing_config_get_binning_size(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the size of the area to bin, 2 would bin a 2x2 area
 * @param ptr Pointer to class
 * @param size The bin size
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_binning_size(tof_processing_config_t ptr, size_t size, tof_error_t *error);

/** Get the binning mode to use
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The binning mode
 */
TOF_EXPORT enum tof_binning_mode tof_processing_config_get_binning_mode(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the binning mode to use
 * @param ptr Pointer to class
 * @param mode The binning mode
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_binning_mode(tof_processing_config_t ptr, enum tof_binning_mode mode, tof_error_t *error);

/** Get The maximum standard deviation to use, only used in the smart binning
 * mode
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The standard deviation
 */
TOF_EXPORT float tof_processing_config_get_binning_sigma(const tof_processing_config_t ptr, tof_error_t *error);

/** Set The maximum standard deviation to use, only used in the smart binning
 * mode
 * @param ptr Pointer to class
 * @param sigma The standard deviation
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_binning_sigma(tof_processing_config_t ptr, float sigma, tof_error_t *error);

/** Get the dimension of the xyz output
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return The dimension
 */
TOF_EXPORT enum tof_xyz_dimension tof_processing_config_get_xyz_dimension(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the dimension of the xyz output
 * @param ptr Pointer to class
 * @param dimension The dimension
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_xyz_dimension(tof_processing_config_t ptr, enum tof_xyz_dimension dimension, tof_error_t *error);

/** Get whether rigid transformation is enabled
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Enabled
 */
TOF_EXPORT bool tof_processing_config_get_rigid_transformation_enabled(const tof_processing_config_t ptr, tof_error_t *error);

/** Set whether rigid transformation is enabled
 * @param ptr Pointer to class
 * @param enabled Enabled
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_rigid_transformation_enabled(tof_processing_config_t ptr, bool enabled, tof_error_t *error);

/** Get the rigid transformation 4x4 matrix
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return[16] 4x4 transformation matrix
 */
TOF_EXPORT float* tof_processing_config_get_rigid_transformation_matrix(const tof_processing_config_t ptr, tof_error_t *error);

/** Set the rigid transformation 4x4 matrix
 * @param ptr Pointer to class
 * @param matrix[16] 4x4 transformation matrix
 * @param error Pointer to error
 */
TOF_EXPORT void tof_processing_config_set_rigid_transformation_matrix(tof_processing_config_t ptr, const float* matrix, tof_error_t *error);

#ifdef __cplusplus
}
#endif

#endif
