import ctypes
from typing import List, Optional
from .__library__ import lib, get_error
from .camera_config import *
from .calibration import *
from .data import *
from .gige_interface import *


class UsbDevice():
    """
    USB device
    """
    def __del__(self):
        if self._ptr:
            lib.tof_usb_device_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, ptr: Optional[ctypes.c_void_p] = None):
        """
        Construct new usb_device class
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        else:
            raise RuntimeError("Can't construct empty class")

    def serial(self) -> str:
        """
        Get serial corresponding with USB device

        Returns
        ----------
        str
            Serial
        """
        return lib.tof_usb_device_serial(self._ptr, get_error()).decode("utf-8")


class UsbInterface():
    """
    Connect to a camera directly using the USB interface
    """
    def __del__(self):
        if self._ptr:
            lib.tof_usb_interface_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, usb_device: Optional[UsbDevice] = None, ptr: Optional[ctypes.c_void_p] = None):
        """
        Construct new usb_interface class
        ----------

        Create USB camera from a usb device

        Parameters
        ----------
        usb_device : UsbDevice
                USB device
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        elif usb_device != None:
            self._ptr = lib.tof_usb_interface_new(usb_device._ptr, get_error())
        else:
            raise RuntimeError("Can't construct empty class")

    def isConnected(self) -> bool:
        """
        Check whether a connection is still maintained with the camera

        Returns
        ----------
        bool
            Is connected
        """
        return lib.tof_usb_interface_is_connected(self._ptr, get_error())


    def testStreamSpeed(self) -> float:
        """
        Tests the connection speed of the USB port. This is useful to check whether
        a hub or USB port works well. The maximum speed that the camera is able to
        transmit is ~3.0 Gbps.

        Returns
        ----------
        float
            Connection speed in Gpbs (bits)
        """
        return lib.tof_usb_interface_test_stream_speed(self._ptr, get_error())


    def downloadConfiguration(self) -> CameraConfig:
        """
        Download camera configuration from the camera

        Returns
        ----------
        CameraConfig
            Camera configuration
        """
        return CameraConfig(ptr = lib.tof_usb_interface_download_configuration(self._ptr, get_error()))


    def downloadCalibration(self) -> Calibration:
        """
        Download calibration from the camera

        Returns
        ----------
        Calibration
            Calibration
        """
        return Calibration(ptr = lib.tof_usb_interface_download_calibration(self._ptr, get_error()))


    def uploadConfiguration(self, config: CameraConfig) -> None:
        """
        Upload camera configuration to the camera

        Parameters
        ----------
        config : CameraConfig
            Camera configuration
        """
        return lib.tof_usb_interface_upload_configuration(self._ptr, config._ptr, get_error())


    def depthStreamCapable(self, stream: DepthStreamType) -> bool:
        """
        Check whether this camera is capable of streaming the specified type

        Parameters
        ----------
        stream : DepthStreamType
            Depth stream type

        Returns
        ----------
        bool
            Is capable
        """
        return lib.tof_usb_interface_depth_stream_capable(self._ptr, stream, get_error())


    def startDepthStream(self, stream: DepthStreamType) -> None:
        """
        Start the depth stream

        Parameters
        ----------
        stream : DepthStreamType
            Depth stream type
        """
        return lib.tof_usb_interface_start_depth_stream(self._ptr, stream, get_error())


    def stopDepthStream(self) -> None:
        """
        Stop the depth stream
        """
        return lib.tof_usb_interface_stop_depth_stream(self._ptr, get_error())


    def hasDepthData(self) -> bool:
        """
        Returns true if a depth frame is available

        Returns
        ----------
        bool
            Depth frame available
        """
        return lib.tof_usb_interface_has_depth_data(self._ptr, get_error())


    def getDepthData(self) -> Data:
        """
        Get a depth frame

        Returns
        ----------
        Data
            Depth frame
        """
        return Data(ptr = lib.tof_usb_interface_get_depth_data(self._ptr, get_error()))


    def imageStreamCapable(self) -> bool:
        """
        Check whether camera supports an image stream

        Returns
        ----------
        bool
            Camera supports an image stream
        """
        return lib.tof_usb_interface_image_stream_capable(self._ptr, get_error())


    def startImageStream(self) -> None:
        """
        Start image stream
        """
        return lib.tof_usb_interface_start_image_stream(self._ptr, get_error())


    def stopImageStream(self) -> None:
        """
        Stop image stream
        """
        return lib.tof_usb_interface_stop_image_stream(self._ptr, get_error())


    def hasImageData(self) -> bool:
        """
        Check whether image frame is available

        Returns
        ----------
        bool
            Image frame available
        """
        return lib.tof_usb_interface_has_image_data(self._ptr, get_error())


    def getImageData(self) -> Data:
        """
        Get image frame

        Returns
        ----------
        Data
            Image data
        """
        return Data(ptr = lib.tof_usb_interface_get_image_data(self._ptr, get_error()))


    def getUserPointerCapacity(self) -> int:
        """
        Get the capacity for amount of user pointers that can be registered

        Returns
        ----------
        int
            User pointer capacity
        """
        return lib.tof_usb_interface_get_user_pointer_capacity(self._ptr, get_error())


    def setUserPointerCapacity(self, capacity: int) -> None:
        """
        Set the capacity for amount of user pointers that can be registered

        Parameters
        ----------
        capacity : int
            User pointer capacity
        """
        return lib.tof_usb_interface_set_user_pointer_capacity(self._ptr, capacity, get_error())


    def version(self) -> str:
        """
        Get the tof library version running on the camera

        Returns
        ----------
        str
            Version
        """
        return lib.tof_usb_interface_version(self._ptr, get_error()).decode("utf-8")


    def softwareTriggerCapable(self) -> bool:
        """
        Check whether the camera is capable of software trigger

        Returns
        ----------
        bool
            Software trigger capable
        """
        return lib.tof_usb_interface_software_trigger_capable(self._ptr, get_error())


    def softwareTrigger(self) -> None:
        """
        Software trigger the camera
        """
        return lib.tof_usb_interface_software_trigger(self._ptr, get_error())


def usbDeviceDiscover() -> List[UsbDevice]:
    """
    Discover all connected chronoptics usb cameras

    Returns
    ----------
    List[UsbDevice]
        The usb devices
    """
    size = lib.tof_usb_device_discover(ctypes.c_void_p(), 0, get_error())
    array = (ctypes.c_void_p * size)()
    lib.tof_usb_device_discover(array, size, get_error())
    return [UsbDevice(ptr = item) for item in array]

def usbDeviceDiscoverOne() -> UsbDevice:
    """
    Discover one chronoptics usb camera

    Returns
    ----------
    UsbDevice
        A USB device
    """
    return UsbDevice(ptr = lib.tof_usb_device_discover_one(get_error()))


def usbDeviceFind(serial: str) -> UsbDevice:
    """
    Discover usb camera with the given serial

    Parameters
    ----------
    serial : str
        Serial of the camera to find

    Returns
    ----------
    UsbDevice
        A USB device
    """
    return UsbDevice(ptr = lib.tof_usb_device_find(serial.encode("utf-8"), get_error()))


